/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: Job_impl.java 4607 2006-11-30 09:10:11Z jz $

package sos.spooler;

/*+
 * Oberklasse für die Implementierung eines Startskripts oder eines Jobs.
 * 
 * Der Methoden eines Jobs werden in folgender Reihenfolge aufgerufen.
 * <pre>
 *     spooler_init()
 *         spooler_open()
 *             spooler_process()
 *             spooler_process()
 *             ...
 *         spooler_close()
 *         spooler_on_success() oder spooler_on_error()
 *     spooler_exit()
 * </pre>
 * 
 * <p>
 *      Keine dieser Methoden muss implementiert werden. In der Regel wird wenigstens spooler_process() implementiert.
 * </p>
 * <p>
 *  Ein Fehler beim Ausführen des Job-Skripts während des Ladens oder in spooler_init() führt zum Aufruf von spooler_on_error(). Der Job wird gestoppt. spooler_exit() wird gerufen (obwohl spooler_init() nicht gerufen worden ist!) und die Scripting Engine wird geschlossen.
 * </p>
 * <p>
 *  spooler_on_error() muss also auch mit Fehlern umgehen, die beim Laden oder in spooler_init() auftreten. 
 * </p>
 * <p>
 *  spooler_exit() wird gerufen, auch wenn spooler_init() nicht gerufen worden ist.
 * </p>
 */
/** 
 * @author Joacim Zschimmer, Zschimmer GmbH
 * @version $Revision: 4607 $
 */

public class Job_impl
{
    protected Job_impl()
    {
    }
    
    
    public final String  spooler_api_version()  
    { 
        return "2.0.160.4605 (2006-11-23)"; 
    }
    
    
    /*+ Der Scheduler ruft diese Methode nach dem Konstruktor und vor {@link #spooler_open()} genau einmal auf. 
      * Gegenstück ist {@link #spooler_exit()}. Die Methode ist geeignet, um die Task zu initialisieren 
      * (z.B. um eine Datenbank-Verbindung aufzubauen).
      * 
      * @return false beendet die Task. Der Scheduler setzt mit spooler_exit() fort.
      * Wenn die Task einen Auftrag verarbeiten soll, wird die Task mit Fehler beendet,
      * was den Job stoppt (wenn nicht {@link Job#set_delay_after_error(int,double)} gesetzt ist).
      */
    
    public boolean  spooler_init        ()      throws Exception  { return true; }

    

    /*+ Wir als allerletzte Methode gerufen, bevor das Java-Objekt verworfen wird. 
      * Hier kann z.B. eine Datenbank-Verbindung geschlossen werden. 
      */
    
    public void     spooler_exit        ()      throws Exception  {}


    
    /*+ Wird zu Beginn einer Task gerufen. 
      * Die Methode wird direkt nach {@link #spooler_init()} gerufen, es gibt derzeit keinen Unterschied.
      * Gegenstück ist {@link #spooler_close()}.
      * @return false beendet die Task. Der Scheduler setzt mit spooler_close() fort.
      */

    public boolean  spooler_open        ()      throws Exception  { return true; }


    
    /*+ Wird am Ende eines Joblaufs gerufen.
      * Gegenstück zu {@link #spooler_open()}.
      */

    public void     spooler_close       ()      throws Exception  {}


    /*+ Führt einen Jobschritt aus.
      * Gegenstück ist {@link #spooler_exit()}.
      * Die Default-Implementierung gibt false zurück (versetzt aber einen Auftrag in den Folgezustand).
      * 
      * @return bei &lt;job order="no">: false beendet den Joblauf.
      * Bei &lt;job order="true">: false versetzt den Auftrag in den Fehlerzustand (s. {@link Job_chain_node}).
      */

    public boolean  spooler_process     ()      throws Exception  
    { 
        //spooler_method_not_implemented(); 
        return false; 
    }


    /*+ Wird als letzte Funktion eines Joblaufs gerufen, wenn ein Fehler aufgetreten ist 
      * (nach {@link #spooler_close()}, vor {@link #spooler_exit()}).
      */

    public void     spooler_on_error    ()      throws Exception  {}

    
    /*+ Wird als letzte Funktion einer fehlerlosen Task gerufen (nach {@link #spooler_close()}, vor {@link #spooler_exit()}).
      */

    public void     spooler_on_success  ()      throws Exception  {}



    //private native void spooler_method_not_implemented();


    
    /*+ Zum Protokollieren.
     * 
     * <p><br/><b>Beispiel</b>
     * <pre>
     *     spooler_log.debug( "Eine Debug-Ausgabe" );
     * </pre>
     * 
     * @see Log#debug(String)
     */
    public Log      spooler_log;

    
    /*+ Das Objekt der Task.
     * 
     * <p><br/><b>Beispiel</b>
     * <pre>
     *     spooler_log.debug( "Meine Task-Id ist " + spooler_task.id() );
     * </pre>
     * 
     * <p><br/><b>Beispiel für JavaScript</b>
     * <pre>
     *     spooler_log.debug( "Meine Task-Id ist " + spooler_task.id );
     * </pre>
     * 
     * @see Task#id()
     */
    public Task     spooler_task;


    
    /*+ Das Objekt des Jobs.
     * 
     * <p><br/><b>Beispiel</b>
     * <pre>
     *     spooler_log.debug( "Der Jobname ist " + spooler_job.name() );
     * </pre>
     * 
     * <p><br/><b>Beispiel für JavaScript</b>
     * <pre>
     *     spooler_log.debug( "Der Jobname ist " + spooler_job.name );
     * </pre>
     * 
     * @see Job#name()
     */
    public Job      spooler_job;

    
    //public Thread   spooler_thread;

    
    /*+ Das Objekt des Schedulers.
     * 
     * <p><br/><b>Beispiel</b>
     * <pre>
     *     spooler_log.debug( "Das Startverzeichnis des Schedulers ist " + spooler.directory() );
     * </pre>
     * 
     * <p><br/><b>Beispiel für JavaScript</b>
     * <pre>
     *     spooler_log.debug( "Das Startverzeichnis des Schedulers ist " + spooler.directory );
     * </pre>
     * 
     * @see Spooler#directory()
     */
    public Spooler  spooler;
}
